@@ -21,6 +21,7 @@
 */
 
 #include <config.h>
+#include <libgnome/gnome-i18n.h>
 #include <unistd.h>
 #include <string.h>
 #include <glade/glade.h>
@@ -253,10 +254,10 @@ get_sample_html (GtkHTMLEditImageProperties *d, gboolean insert)
 		html   = g_strconcat (body, image, NULL);
 	} else {
 		html   = g_strconcat (body,
-				      _("The quick brown fox jumped over the lazy dog."),
+				      _("The quick brown fox jumps over the lazy dog."),
 				      " ",
 				      image,
-				      _("The quick brown fox jumped over the lazy dog."),
+				      _("The quick brown fox jumps over the lazy dog."),
 				      NULL);
 	}
 
