@@ -20,6 +20,9 @@
     Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+#include <libgnome/gnome-i18n.h>
+#include <string.h>
 #include <glade/glade.h>
 #include <gal/widgets/widget-color-combo.h>
 
@@ -36,7 +39,6 @@
 #include "htmltable.h"
 #include "htmlsettings.h"
 
-#include "config.h"
 #include "properties.h"
 #include "table.h"
 #include "utils.h"
@@ -56,7 +58,7 @@ typedef struct
 
 	gboolean   has_bg_pixmap;
 	gboolean   changed_bg_pixmap;
-	gchar     *bg_pixmap;
+	const gchar *bg_pixmap;
 	GtkWidget *entry_bg_pixmap;
 	GtkWidget *check_bg_pixmap;
 
@@ -362,7 +364,7 @@ set_has_bg_pixmap (GtkWidget *check, GtkHTMLEditTableProperties *d)
 }
 
 static void
-changed_bg_color (GtkWidget *w, GdkColor *color, gboolean by_user, GtkHTMLEditTableProperties *d)
+changed_bg_color (GtkWidget *w, GdkColor *color, gboolean custom, gboolean by_user, gboolean is_default, GtkHTMLEditTableProperties *d)
 {
 	/* If the color was changed programatically there's not need to set things */
 	if (!by_user)
@@ -523,7 +525,7 @@ table_widget (GtkHTMLEditTableProperties *d)
 	GtkWidget *table_page;
 	GladeXML *xml;
 
-	xml = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "table_page");
+	xml = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "table_page", NULL);
 	if (!xml)
 		g_error (_("Could not load glade file."));
 
@@ -533,61 +535,54 @@ table_widget (GtkHTMLEditTableProperties *d)
 	html_color_alloc (color, d->cd->html->engine->painter);
 	d->combo_bg_color = color_combo_new (NULL, _("Automatic"), &color->color,
 					     color_group_fetch ("table_bg_color", d->cd));
-        gtk_signal_connect (GTK_OBJECT (d->combo_bg_color), "changed", GTK_SIGNAL_FUNC (changed_bg_color), d);
+        g_signal_connect (d->combo_bg_color, "color_changed", G_CALLBACK (changed_bg_color), d);
 	gtk_table_attach (GTK_TABLE (glade_xml_get_widget (xml, "bg_table")),
 			  d->combo_bg_color,
 			  1, 2, 0, 1, 0, 0, 0, 0);
 
 	d->check_bg_color  = glade_xml_get_widget (xml, "check_table_bg_color");
-	gtk_signal_connect (GTK_OBJECT (d->check_bg_color), "toggled", set_has_bg_color, d);
+	g_signal_connect (d->check_bg_color, "toggled", G_CALLBACK (set_has_bg_color), d);
 	d->check_bg_pixmap = glade_xml_get_widget (xml, "check_table_bg_pixmap");
-	gtk_signal_connect (GTK_OBJECT (d->check_bg_pixmap), "toggled", set_has_bg_pixmap, d);
+	g_signal_connect (d->check_bg_pixmap, "toggled", G_CALLBACK (set_has_bg_pixmap), d);
 	d->entry_bg_pixmap = glade_xml_get_widget (xml, "entry_table_bg_pixmap");
 
-	d->disable_change = TRUE;
-	/* fix for broken gnome-libs, could be removed once gnome-libs are fixed */
-	gnome_entry_load_history (GNOME_ENTRY (gnome_pixmap_entry_gnome_entry (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap))));
-	our_gnome_pixmap_entry_set_last_dir (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap));
-	d->disable_change = FALSE;
-
-	gtk_signal_connect (GTK_OBJECT (gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap))),
-			    "changed", GTK_SIGNAL_FUNC (changed_bg_pixmap), d);
-
+	g_signal_connect (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (d->entry_bg_pixmap)),
+			  "changed", G_CALLBACK (changed_bg_pixmap), d);
 
 	d->spin_spacing = glade_xml_get_widget (xml, "spin_spacing");
-	gtk_signal_connect (GTK_OBJECT (d->spin_spacing), "changed", changed_spacing, d);
+	g_signal_connect (d->spin_spacing, "value_changed", G_CALLBACK (changed_spacing), d);
 	d->spin_padding = glade_xml_get_widget (xml, "spin_padding");
-	gtk_signal_connect (GTK_OBJECT (d->spin_padding), "changed", changed_padding, d);
+	g_signal_connect (d->spin_padding, "value_changed", G_CALLBACK (changed_padding), d);
 	d->spin_border  = glade_xml_get_widget (xml, "spin_border");
-	gtk_signal_connect (GTK_OBJECT (d->spin_border), "changed", changed_border, d);
+	g_signal_connect (d->spin_border, "value_changed", G_CALLBACK (changed_border), d);
 	UPPER_FIX (padding);
 	UPPER_FIX (spacing);
 	UPPER_FIX (border);
 
 	d->option_align = glade_xml_get_widget (xml, "option_table_align");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_align))), "selection-done",
-			    changed_align, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_align)), "selection-done",
+			  G_CALLBACK (changed_align), d);
 
 	d->spin_width   = glade_xml_get_widget (xml, "spin_table_width");
-	gtk_signal_connect (GTK_OBJECT (d->spin_width), "changed", changed_width, d);
+	g_signal_connect (d->spin_width, "value_changed", G_CALLBACK (changed_width), d);
 	UPPER_FIX (width);
 	d->check_width  = glade_xml_get_widget (xml, "check_table_width");
-	gtk_signal_connect (GTK_OBJECT (d->check_width), "toggled", set_has_width, d);
+	g_signal_connect (d->check_width, "toggled", G_CALLBACK (set_has_width), d);
 	d->option_width = glade_xml_get_widget (xml, "option_table_width");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_width))), "selection-done",
-			    changed_width_percent, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_width)), "selection-done",
+			  G_CALLBACK (changed_width_percent), d);
 
 	d->spin_cols = glade_xml_get_widget (xml, "spin_table_columns");
-	gtk_signal_connect (GTK_OBJECT (d->spin_cols), "changed", changed_cols, d);
+	g_signal_connect (d->spin_cols, "value_changed", G_CALLBACK (changed_cols), d);
 	d->spin_rows = glade_xml_get_widget (xml, "spin_table_rows");
-	gtk_signal_connect (GTK_OBJECT (d->spin_rows), "changed", changed_rows, d);
+	g_signal_connect (d->spin_rows, "value_changed", G_CALLBACK (changed_rows), d);
 	UPPER_FIX (cols);
 	UPPER_FIX (rows);
 
-	gtk_box_pack_start (GTK_BOX (table_page), sample_frame (&d->sample), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (table_page), sample_frame (&d->sample), TRUE, TRUE, 0);
 
 	gtk_widget_show_all (table_page);
-        gdk_color_alloc (gdk_window_get_colormap (d->cd->html->engine->window), &d->bg_color);
+        /* RM2 gdk_color_alloc (gdk_window_get_colormap (d->cd->html->engine->window), &d->bg_color); */
 	gnome_pixmap_entry_set_preview (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap), FALSE);
 
 	return table_page;
@@ -602,7 +597,7 @@ fill_templates (GtkHTMLEditTableProperties *d)
 	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_template));
 
 	for (i = 0; i < TEMPLATES; i ++)
-		gtk_menu_append (GTK_MENU (menu), gtk_menu_item_new_with_label (_(table_templates [i].name)));
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_menu_item_new_with_label (_(table_templates [i].name)));
 	gtk_menu_set_active (GTK_MENU (menu), 0);
 	gtk_container_remove (GTK_CONTAINER (menu), gtk_menu_get_active (GTK_MENU (menu)));
 }
@@ -614,43 +609,43 @@ table_insert_widget (GtkHTMLEditTableProperties *d)
 	GladeXML *xml;
 
 	d->insert = TRUE;
-	xml       = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "table_insert_page");
+	xml       = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "table_insert_page", NULL);
 	if (!xml)
 		g_error (_("Could not load glade file."));
 
 	table_page = glade_xml_get_widget (xml, "table_insert_page");
 
 	d->spin_cols = glade_xml_get_widget (xml, "spin_table_columns");
-	gtk_signal_connect (GTK_OBJECT (d->spin_cols), "changed", changed_cols, d);
+	g_signal_connect (d->spin_cols, "value_changed", G_CALLBACK (changed_cols), d);
 	d->spin_rows = glade_xml_get_widget (xml, "spin_table_rows");
-	gtk_signal_connect (GTK_OBJECT (d->spin_rows), "changed", changed_rows, d);
+	g_signal_connect (d->spin_rows, "value_changed", G_CALLBACK (changed_rows), d);
 	UPPER_FIX (cols);
 	UPPER_FIX (rows);
 
 	d->spin_width   = glade_xml_get_widget (xml, "spin_table_width");
 	UPPER_FIX (width);
-	gtk_signal_connect (GTK_OBJECT (d->spin_width), "changed", changed_width, d);
+	g_signal_connect (d->spin_width, "value_changed", G_CALLBACK (changed_width), d);
 	d->check_width  = glade_xml_get_widget (xml, "check_table_width");
-	gtk_signal_connect (GTK_OBJECT (d->check_width), "toggled", set_has_width, d);
+	g_signal_connect (d->check_width, "toggled", G_CALLBACK (set_has_width), d);
 	d->option_width = glade_xml_get_widget (xml, "option_table_width");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_width))), "selection-done",
-			    changed_width_percent, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_width)), "selection-done",
+			  G_CALLBACK (changed_width_percent), d);
 
 	d->spin_spacing = glade_xml_get_widget (xml, "spin_spacing");
-	gtk_signal_connect (GTK_OBJECT (d->spin_spacing), "changed", changed_spacing, d);
+	g_signal_connect (d->spin_spacing, "value_changed", G_CALLBACK (changed_spacing), d);
 	d->spin_padding = glade_xml_get_widget (xml, "spin_padding");
-	gtk_signal_connect (GTK_OBJECT (d->spin_padding), "changed", changed_padding, d);
+	g_signal_connect (d->spin_padding, "value_changed", G_CALLBACK (changed_padding), d);
 	d->spin_border  = glade_xml_get_widget (xml, "spin_border");
-	gtk_signal_connect (GTK_OBJECT (d->spin_border), "changed", changed_border, d);
+	g_signal_connect (d->spin_border, "value_changed", G_CALLBACK (changed_border), d);
 	UPPER_FIX (padding);
 	UPPER_FIX (spacing);
 	UPPER_FIX (border);
 
 	d->option_template = glade_xml_get_widget (xml, "option_table_template");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_template))), "selection-done",
-			    changed_template, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_template)), "selection-done",
+			  G_CALLBACK (changed_template), d);
 	fill_templates (d);
-	gtk_box_pack_start (GTK_BOX (table_page), sample_frame (&d->sample), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (table_page), sample_frame (&d->sample), TRUE, TRUE, 0);
 
 	gtk_widget_show_all (table_page);
 
@@ -663,11 +658,11 @@ set_ui (GtkHTMLEditTableProperties *d)
 	d->disable_change = TRUE;
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (d->check_bg_color), d->has_bg_color);
-	gdk_color_alloc (gdk_window_get_colormap (GTK_WIDGET (d->cd->html)->window), &d->bg_color);
+	/* RM2 gdk_color_alloc (gdk_window_get_colormap (GTK_WIDGET (d->cd->html)->window), &d->bg_color); */
 	color_combo_set_color (COLOR_COMBO (d->combo_bg_color), &d->bg_color);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (d->check_bg_pixmap), d->has_bg_pixmap);
-	gtk_entry_set_text (GTK_ENTRY (gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap))),
+	gtk_entry_set_text (GTK_ENTRY (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (d->entry_bg_pixmap))),
 			    d->bg_pixmap);
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (d->spin_spacing), d->spacing);
@@ -780,7 +775,7 @@ table_insert (GtkHTMLControlData *cd, gpointer *set_data)
 	return rv;
 }
 
-void
+gboolean
 table_insert_cb (GtkHTMLControlData *cd, gpointer get_data)
 {
 	GtkHTMLEditTableProperties *d = (GtkHTMLEditTableProperties *) get_data;
@@ -790,16 +785,39 @@ table_insert_cb (GtkHTMLControlData *cd, gpointer get_data)
 
 	position = e->cursor->position + table_templates [d->template].offset;
 	html = get_sample_html (d, FALSE);
-	printf ("INSERT(%d):\n%s\n", d->template, html);
-	gtk_html_insert_html (cd->html, html);
+	/* printf ("INSERT(%d):\n%s\n", d->template, html); */
+	gtk_html_append_html (cd->html, html);
 	g_free (html);
 	html_cursor_jump_to_position (e->cursor, e, position);
+
+	return TRUE;
 }
 
-void
+gboolean
 table_apply_cb (GtkHTMLControlData *cd, gpointer get_data)
 {
 	GtkHTMLEditTableProperties *d = (GtkHTMLEditTableProperties *) get_data;
+	HTMLEngine *e = d->cd->html->engine;
+	gint position;
+
+	position = e->cursor->position;
+
+	if (html_engine_get_table (e) != d->table) {
+		if (html_engine_goto_table_0 (e, d->table))
+			html_cursor_forward (e->cursor, e);
+		if (html_engine_get_table (e) != d->table) {
+			GtkWidget *dialog;
+
+			dialog = gtk_message_dialog_new (GTK_WINDOW (d->cd->properties_dialog->dialog),
+							 GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+							 _("The editted table was removed from the document.\nCannot apply your changes."));
+			gtk_dialog_run (GTK_DIALOG (dialog));
+			gtk_widget_destroy (dialog);
+			html_cursor_jump_to_position (e->cursor, e, position);
+
+			return FALSE;
+		}
+	}
 
 	if (d->changed_bg_color) {
 		html_engine_table_set_bg_color (d->cd->html->engine, d->table, d->has_bg_color ? &d->bg_color : NULL);
@@ -841,6 +859,10 @@ table_apply_cb (GtkHTMLControlData *cd, gpointer get_data)
 		html_engine_table_set_rows (d->cd->html->engine, d->rows);
 		d->changed_rows = FALSE;
 	}
+
+	html_cursor_jump_to_position (e->cursor, e, position);
+
+	return TRUE;
 }
 
 void
