@@ -25,6 +25,7 @@
 #include "htmlclue.h"
 #include "htmlobject.h"
 #include "htmlsearch.h"
+#include "htmltextslave.h"
 #include "htmltype.h"
 
 
@@ -54,6 +55,53 @@ destroy (HTMLObject *o)
 	HTML_OBJECT_CLASS (parent_class)->destroy (o);
 }
 
+static gint
+get_n_children (HTMLObject *o)
+{
+	HTMLObject *cur = HTML_CLUE (o)->head;
+	gint n_children = 0;
+
+	while (cur) {
+		n_children ++;
+		cur = html_object_next_not_slave (cur);
+	}
+
+	return n_children;
+}
+
+static HTMLObject *
+get_child (HTMLObject *o, gint index)
+{
+	HTMLObject *cur = HTML_CLUE (o)->head;
+
+	g_return_val_if_fail (index >= 0, NULL);
+
+	while (cur) {
+		if (!index)
+			break;
+		index --;
+		cur = html_object_next_not_slave (cur);
+	}
+
+	return cur;
+}
+
+static gint
+get_child_index (HTMLObject *self, HTMLObject *child)
+{
+	HTMLObject *cur = HTML_CLUE (self)->head;
+	gint index = 0;
+
+	while (cur) {
+		if (cur == child)
+			return index;
+		index ++;
+		cur = html_object_next_not_slave (cur);
+	}
+
+	return -1;
+}
+
 static guint
 get_recursive_length (HTMLObject *self)
 {
@@ -96,16 +144,19 @@ op_helper (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left,
 	if ((o == NULL) && (last == NULL))
 		return cc;
 
+	if (HTML_IS_TEXT_SLAVE (last))
+		last = html_object_prev_not_slave (last);
+
 	g_assert (o->parent == self);
 	g_assert (last->parent == self);
 
-	while (1) {
+	while (o) {
 		cnext = html_object_next_not_slave (o);
 		child = cut ? html_object_op_cut (o, e,
 					      html_object_get_bound_list (o, from),
 					      html_object_get_bound_list (o, to),
 					      left ? left->next : NULL, right ? right->next : NULL, len)
-			: html_object_op_copy (o, e,
+			: html_object_op_copy (o, cc, e,
 					       html_object_get_bound_list (o, from),
 					       html_object_get_bound_list (o, to), len);
 		if (child)
@@ -120,7 +171,7 @@ op_helper (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left,
 }
 
 static HTMLObject *
-op_copy (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, guint *len)
+op_copy (HTMLObject *self, HTMLObject *parent, HTMLEngine *e, GList *from, GList *to, guint *len)
 {
 	return op_helper (self, e, from, to, NULL, NULL, len, FALSE);
 }
@@ -144,7 +195,7 @@ op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GL
 }
 
 static gboolean
-merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GList *right)
+merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList **left, GList **right, HTMLCursor *cursor)
 {
 	HTMLClue   *clue1, *clue2;
 
@@ -187,6 +238,9 @@ split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, gint lev
 	if (self->parent && HTML_OBJECT_TYPE (self->parent) != HTML_TYPE_TABLE)
 		html_clue_append_after (HTML_CLUE (self->parent), dup, self);
 
+	self->x = 0;
+	self->y = 0;
+
 	*left  = g_list_prepend (*left, self);
 	*right = g_list_prepend (*right, dup);
 
@@ -222,47 +276,20 @@ draw (HTMLObject *o,
 }
 
 static void
-set_max_ascent (HTMLObject *o,
-		HTMLPainter *painter,				
-		gint a)
+set_max_height (HTMLObject *o, HTMLPainter *painter, gint height)
 {
 	HTMLClue *clue = HTML_CLUE (o);
 	HTMLObject *obj;
 
-	if (clue->valign == HTML_VALIGN_MIDDLE) {
-		for (obj = HTML_CLUE (o)->head; obj != 0; obj = obj->next) {
-			obj->y = obj->y + ((a - o->ascent) / 2);
-		}
+	for (obj = HTML_CLUE (o)->head; obj != 0; obj = obj->next) {
+		html_object_set_max_height (obj, painter, height);
+		if (clue->valign == HTML_VALIGN_MIDDLE)
+			obj->y = obj->y + ((height - o->ascent) / 2);
+		else if (clue->valign == HTML_VALIGN_BOTTOM)
+			obj->y = obj->y + height - o->ascent;
 	}
 
-	else if (clue->valign == HTML_VALIGN_BOTTOM) {
-		for (obj = HTML_CLUE (o)->head; obj != 0; obj = obj->next) {
-			obj->y = obj->y + a - o->ascent;
-		}
-	}
-
-	o->ascent = a;
-}
-
-static void
-set_max_descent (HTMLObject *o, HTMLPainter *painter, gint d)
-{
-	HTMLClue *clue = HTML_CLUE (o);
-	HTMLObject *obj;
-	
-	if (clue->valign == HTML_VALIGN_MIDDLE) {
-		for (obj = clue->head; obj != 0; obj = obj->next) {
-			obj->y = obj->y + ((d - o->descent) / 2);
-		}
-	}
-	else if (clue->valign == HTML_VALIGN_BOTTOM) {
-		for (obj = clue->head; obj != 0; obj = obj->next) {
-			obj->y = obj->y + d - o->descent;
-		}
-	}
-	
-	o->descent = d;
-
+	o->ascent = height;
 }
 
 static void
@@ -279,8 +306,7 @@ reset (HTMLObject *clue)
 }
 
 static gboolean
-calc_size (HTMLObject *o,
-	   HTMLPainter *painter)
+calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 {
 	gboolean changed;
 
@@ -294,7 +320,7 @@ calc_size (HTMLObject *o,
 	changed = FALSE;
 
 	while (HTML_CLUE (o)->curr != NULL) {
-		changed |= html_object_calc_size (HTML_CLUE (o)->curr, painter);
+		changed |= html_object_calc_size (HTML_CLUE (o)->curr, painter, changed_objs);
 		HTML_CLUE (o)->curr = HTML_CLUE (o)->curr->next;
 	}
 
@@ -492,6 +518,7 @@ get_right_clear (HTMLClue *o, gint y)
 
 static void
 find_free_area (HTMLClue *clue,
+		HTMLPainter *painter,
 		gint y,
 		gint width, gint height,
 		gint indent, gint *y_pos,
@@ -503,7 +530,7 @@ find_free_area (HTMLClue *clue,
 }
 
 static void
-append_right_aligned (HTMLClue *clue, HTMLClue *aclue)
+append_right_aligned (HTMLClue *clue, HTMLPainter *painter, HTMLClue *aclue, gint *lmargin, gint *rmargin, gint indent)
 {
 	/* This needs to be implemented in the subclasses.  */
 	g_warning ("`%s' does not implement `append_right_aligned()'.",
@@ -601,8 +628,7 @@ html_clue_class_init (HTMLClueClass *klass,
 	object_class->remove_child = remove_child;
 	object_class->split = split;
 	object_class->draw = draw;
-	object_class->set_max_ascent = set_max_ascent;
-	object_class->set_max_descent = set_max_descent;
+	object_class->set_max_height = set_max_height;
 	object_class->reset = reset;
 	object_class->calc_size = calc_size;
 	object_class->calc_preferred_width = calc_preferred_width;
@@ -619,6 +645,9 @@ html_clue_class_init (HTMLClueClass *klass,
 	object_class->head = head;
 	object_class->tail = tail;
 	object_class->get_recursive_length = get_recursive_length;
+	object_class->get_n_children = get_n_children;
+	object_class->get_child = get_child;
+	object_class->get_child_index = get_child_index;
 
 	/* HTMLClue methods.  */
 	klass->get_left_clear = get_left_clear;
@@ -662,33 +691,36 @@ html_clue_get_right_clear (HTMLClue *clue, gint y)
 
 void
 html_clue_find_free_area (HTMLClue *clue,
+			  HTMLPainter *painter,
 			  gint y,
 			  gint width, gint height, gint indent, gint *y_pos,
 			  gint *lmargin, gint *rmargin)
 {
-	(* HC_CLASS (clue)->find_free_area) (clue, y, width, height, indent, y_pos, lmargin, rmargin);
+	(* HC_CLASS (clue)->find_free_area) (clue, painter, y, width, height, indent, y_pos, lmargin, rmargin);
 }
 
 void
-html_clue_append_right_aligned (HTMLClue *clue, HTMLClue *aclue)
+html_clue_append_right_aligned (HTMLClue *clue, HTMLPainter *painter,
+				HTMLClue *aclue, gint *lmargin, gint *rmargin, gint indent)
 {
 	g_assert (clue != NULL);
 	g_assert (aclue != NULL);
 
 	html_object_change_set (HTML_OBJECT (clue), HTML_OBJECT (aclue)->change);
 
-	(* HC_CLASS (clue)->append_right_aligned) (clue, aclue);
+	(* HC_CLASS (clue)->append_right_aligned) (clue, painter, aclue, lmargin, rmargin, indent);
 }
 
 void
-html_clue_append_left_aligned (HTMLClue *clue, HTMLClue *aclue)
+html_clue_append_left_aligned (HTMLClue *clue, HTMLPainter *painter,
+			       HTMLClue *aclue, gint *lmargin, gint *rmargin, gint indent)
 {
 	g_assert (clue != NULL);
 	g_assert (aclue != NULL);
 
 	html_object_change_set (HTML_OBJECT (clue), HTML_OBJECT (aclue)->change);
 
-	(* HC_CLASS (clue)->append_left_aligned) (clue, aclue);
+	(* HC_CLASS (clue)->append_left_aligned) (clue, painter, aclue, lmargin, rmargin, indent);
 }
 
 gboolean
@@ -741,6 +773,8 @@ html_clue_append_after (HTMLClue *clue,
 	HTMLObject *tail;
 
 	g_return_if_fail (o != NULL);
+	g_return_if_fail (html_object_is_clue (HTML_OBJECT (clue)));
+
 	if (where == NULL) {
 		html_clue_prepend (clue, o);
 		return;
@@ -778,6 +812,7 @@ html_clue_append (HTMLClue *clue,
 	HTMLObject *tail;
 
 	g_return_if_fail (clue != NULL);
+	g_return_if_fail (html_object_is_clue (HTML_OBJECT (clue)));
 	g_return_if_fail (o != NULL);
 
 	html_object_change_set (HTML_OBJECT (clue), o->change);
@@ -864,3 +899,21 @@ html_clue_remove (HTMLClue *clue,
 	o->prev = NULL;
 	o->next = NULL;
 }
+
+void
+html_clue_remove_text_slaves (HTMLClue *clue)
+{
+	HTMLObject *p;
+	HTMLObject *pnext;
+
+	g_return_if_fail (clue != NULL);
+
+	for (p = clue->head; p != NULL; p = pnext) {
+		pnext = p->next;
+
+		if (HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTSLAVE) {
+			html_clue_remove (clue, p);
+			html_object_destroy (p);
+		}
+	}
+}
