@@ -56,11 +56,14 @@ struct _HTMLImageFactory {
 	HTMLEngine *engine;
 	GHashTable *loaded_images;
 	GdkPixbuf  *missing;
+	gboolean    animate;
 };
 
 
 #define DEFAULT_SIZE 48
 #define STRDUP_HELPER(i,j) if (i != j) {char *tmp = g_strdup (j); g_free(i); i = tmp;}
+
+#define DA(x)
 
 HTMLImageClass html_image_class;
 static HTMLObjectClass *parent_class = NULL;
@@ -343,9 +346,18 @@ draw_plain (HTMLObject *o, HTMLPainter *p, gint x, gint y, gint width, gint heig
 	HTMLImage *img = HTML_IMAGE (o);
 
 	if (img->alt && *img->alt) {
-		html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLTextColor)->color);
+
 		/* FIXME: cache items and glyphs? */
-		html_painter_draw_text (p, o->x + tx, o->y + ty - o->ascent, img->alt, g_utf8_strlen (img->alt, -1), NULL, NULL, 0, 0);
+		if (o->selected) {
+			html_painter_set_pen (p, &html_colorset_get_color_allocated
+					      (p, p->focus ? HTMLHighlightColor : HTMLHighlightNFColor)->color);
+			html_painter_fill_rect (p, o->x + tx, o->y + ty - o->ascent, o->width, o->ascent + o->descent);
+			html_painter_set_pen (p, &html_colorset_get_color_allocated
+					      (p, p->focus ? HTMLHighlightTextColor : HTMLHighlightTextNFColor)->color);
+		} else { 
+			html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLTextColor)->color);
+		}
+		html_painter_draw_text (p, o->x + tx, o->y + ty, img->alt, g_utf8_strlen (img->alt, -1), NULL, NULL, 0, 0);
 	}
 }
 
@@ -483,8 +495,8 @@ html_image_resolve_image_url (GtkHTML *html, gchar *image_url)
 
 		if (oarg) {
 			if (G_VALUE_TYPE (oarg) == G_TYPE_STRING)
-				url = (gchar *) g_value_get_string (oarg);
-			g_value_unset (iarg);	
+				url = (gchar *) g_strdup (g_value_get_string (oarg));
+			g_value_unset (oarg);	
 			g_free (oarg);
 		}
 		g_value_unset (iarg);
@@ -1079,7 +1091,7 @@ html_image_pointer_queue_animation (HTMLImagePointer *ip)
 {
 	gint delay = gdk_pixbuf_animation_iter_get_delay_time (ip->iter);
 
-	if (delay >= 0 && !ip->animation_timeout) {
+	if (delay >= 0 && !ip->animation_timeout && ip->factory && ip->factory->animate) {
 		ip->animation_timeout = g_timeout_add (delay, 
 						       (GtkFunction) html_image_pointer_run_animation, 
 						       (gpointer) ip);
@@ -1151,6 +1163,7 @@ html_image_factory_new (HTMLEngine *e)
 	retval->engine = e;
 	retval->loaded_images = g_hash_table_new (g_str_hash, g_str_equal);
 	retval->missing = NULL;
+	retval->animate = TRUE;
 
 	return retval;
 }
@@ -1325,6 +1338,7 @@ html_image_factory_register (HTMLImageFactory *factory, HTMLImage *i, const char
 
 	if (!ip) {
 		ip = html_image_pointer_new (url, factory);
+		g_hash_table_insert (factory->loaded_images, ip->url, ip);
 		if (*url) {
 			g_signal_connect (G_OBJECT (ip->loader), "area_prepared",
 					  G_CALLBACK (html_image_factory_area_prepared),
@@ -1333,8 +1347,6 @@ html_image_factory_register (HTMLImageFactory *factory, HTMLImage *i, const char
 			g_signal_connect (G_OBJECT (ip->loader), "area_updated",
 					  G_CALLBACK (html_image_factory_area_updated),
 					  ip);
-
-			g_hash_table_insert (factory->loaded_images, ip->url, ip);
 			stream = html_image_pointer_load (ip);
 		}
 	} else {
@@ -1403,9 +1415,43 @@ stop_anim (gpointer key, gpointer value, gpointer user_data)
 void
 html_image_factory_stop_animations (HTMLImageFactory *factory)
 {
+	DA (g_warning ("stop animations");)
 	g_hash_table_foreach (factory->loaded_images, stop_anim, NULL);
 }
 
+static void
+start_anim (gpointer key, gpointer value, gpointer user_data)
+{
+	HTMLImagePointer *ip = value;
+	html_image_pointer_start_animation (ip);
+}
+
+void
+html_image_factory_start_animations (HTMLImageFactory *factory)
+{
+	DA (g_warning ("start animations");)
+	g_hash_table_foreach (factory->loaded_images, start_anim, NULL);
+}
+
+gboolean
+html_image_factory_get_animate (HTMLImageFactory *factory)
+{
+	return factory->animate;
+}
+
+void
+html_image_factory_set_animate (HTMLImageFactory *factory, gboolean animate)
+{
+	if (animate != factory->animate) {
+		factory->animate = animate;
+
+		if (animate)
+			html_image_factory_start_animations (factory);
+		else 
+			html_image_factory_stop_animations (factory);
+	}
+}
+
 static gboolean
 move_image_pointers (gpointer key, gpointer value, gpointer data)
 {
